How to use the Debian OpenCV java packages
----

Please see
  http://docs.opencv.org/doc/tutorials/introduction/desktop_java/java_dev_intro.html

  ant -DocvJarDir=/usr/share/java/ -DocvLibDir=/usr/lib/
